package org.implab.gradle.containers.tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.file.RegularFile;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.OutputFile;
import org.gradle.api.tasks.TaskExecutionException;
import org.implab.gradle.Utils;
import org.implab.gradle.containers.ImageTag;

public abstract class SaveImage extends CliTask {

    @Input
    public abstract Property<ImageTag> getImage();

    @OutputFile
    public Provider<RegularFile> getArchiveFile() {
        return getDestinationDirectory().file(getArchiveFileName());
    }

    @Internal
    public abstract DirectoryProperty getDestinationDirectory();

    @Internal
    public abstract Property<String> getArchiveFileName();

    @Internal
    public abstract Property<String> getArchiveBaseName();

    @Internal
    public abstract Property<String> getArchiveVersion();

    @Internal
    public abstract Property<String> getArchiveClassifier();

    @Internal
    public abstract Property<String> getArchiveExtension();

    public SaveImage() {
        getArchiveFileName().convention(getProject().provider(this::conventionalArchiveFileName));
        getArchiveBaseName().convention(getProject().provider(this::conventionalArchiveBaseName));
        getArchiveVersion().convention(getProject().provider(() -> getProject().getVersion()).map(x -> x.toString()));
        getDestinationDirectory().convention(getProject().getLayout().getBuildDirectory());
        getArchiveExtension().convention("tar");
    }

    private String conventionalArchiveBaseName() {
        ArrayList<String> parts = new ArrayList<>();
        Optional.of(getProject().getGroup()).map(x -> x.toString()).ifPresent(parts::add);
        parts.add(getProject().getName());
        return String.join("-", parts);
    }

    private String conventionalArchiveFileName() {
        ArrayList<String> parts = new ArrayList<>();

        if (getArchiveBaseName().isPresent())
            parts.add(getArchiveBaseName().get());

        if (getArchiveVersion().isPresent())
            parts.add(getArchiveVersion().get());

        if (getArchiveClassifier().isPresent())
            parts.add(getArchiveClassifier().get());

        if (parts.size() == 0)
            throw new TaskExecutionException(this, new Exception("The archiveFileName ism't specified"));

        return String.join("-", parts) + "." + getArchiveExtension().get();
    }

    @Override
    protected void preparingCommand(List<String> commandLine) {
        super.preparingCommand(commandLine);

        commandLine.add("save");
        commandLine.add(getImage().get().toString());
    }

    @Override
    protected void processStarted(Process p) {
        Utils.redirectIO(p.getInputStream(), getArchiveFile().get().getAsFile());
    }
}
